<?php
/**
 * Plugin Name: EP Webshop
 * Description: eCommerce rješenje za WordPress.
 * Version: 1.3.0
 * Author: Elektronika Perhoč
 * Author URI: https://ep.hr
 */

use Ep\WordPress\Service\WordPressServiceInterface;
use Psr\Container\ContainerInterface;

define('EP_WEBSHOP_PLUGIN', __FILE__);
define('EP_WEBSHOP_DIR', __DIR__);

require_once ABSPATH . 'wp-admin/includes/plugin.php';
$epWebshopPluginData = get_plugin_data(__FILE__, false, false);
define('EP_WEBSHOP_VERSION', $epWebshopPluginData['Version']);
unset($epWebshopPluginData);

require_once __DIR__ . '/compat/flatsome.php';
require_once __DIR__ . '/i18n/srlatin.php';

if (defined('ABSPATH')) {
	if (function_exists('get_option')) {
		define('EP_WEBSHOP_URL', rtrim(plugin_dir_url(EP_WEBSHOP_PLUGIN), '/'));
	}

	if (!defined('CACHE_DIR') && function_exists('get_option')) {
		$uploadDir = wp_get_upload_dir();
		define('CACHE_DIR', sprintf('%s/cache', $uploadDir['basedir']));
	}

	register_activation_hook(EP_WEBSHOP_PLUGIN, 'ep_activate');

	$services = ep_services();

	// Ovo nam treba prije inita
	if ($services->has('WordPressI18n')) {
		$services->get('WordPressI18n');
	}

	if ($services->has('WordPressMetadata')) {
		$services->get('WordPressMetadata');
	}

	if (function_exists('get_option')) {
		$gathSettings = get_option('ep_gath', []);

		if (isset($gathSettings['enabled']) && '1' === $gathSettings['enabled']) {
			if (isset($gathSettings['modules']) && is_array($gathSettings['modules'])) {
				if (in_array('products', $gathSettings['modules'])) {
					if ($services->has('GathProductProvider')) {
						$services->get('GathProductProvider');
					}
				}

				if (in_array('stock', $gathSettings['modules'])) {
					if ($services->has('GathStockProvider')) {
						$services->get('GathStockProvider');
					}
				}
			}
		}
	}

	add_action('init', 'ep_init');
	add_action('wp_enqueue_scripts', 'ep_assets');
	add_action('admin_init', 'ep_admin_init');
	add_action('admin_enqueue_scripts', 'ep_admin_assets');
	add_filter('status_header', 'ep_status_code', 9999, 1);
	add_action('widgets_init', 'ep_widgets');

	add_action('plugins_loaded', function() {
		load_plugin_textdomain('ep-webshop', false, basename(dirname(__FILE__)) . '/i18n');
	});
}

/**
 * Ne prihvaćamo opis kategorije iz Gatha.
 */
/** @noinspection PhpUnusedParameterInspection */
add_filter('rest_pre_dispatch', function($null, /** @noinspection PhpUnusedParameterInspection */ $server, WP_REST_Request $request) {
	ini_set('display_errors', 0);

	if ($request->get_route() === '/wc/v2/products/categories') {
		$body = json_decode($request->get_body(), true);
		if (isset($body['description'])) {
			unset($body['description']);
			$request->set_body(json_encode($body));
		}
	} else if (preg_match('#/wc/v2/products/attributes/\d+$#', $request->get_route()) && 'PUT' === $_SERVER['REQUEST_METHOD']) {
		$body = json_decode($request->get_body(), true);
		if (!isset($body['slug']) && isset($body['name'])) {
			$body['slug'] = wc_sanitize_taxonomy_name(stripslashes($body['name']));
			$request->set_body(json_encode($body));
		}
	}

	return $null;
}, 10, 3);

/**
 * Inicijalizira cijelu priču.
 */
function ep_init()
{
	$services = ep_services();

	/** @var WordPressServiceInterface $wordpress */
	$wordpress = $services->get('WordPress');
	$wordpress->attach();
}

/**
 * Inicijalizira naše widgete.
 */
function ep_widgets()
{
	register_widget(\Ep\Widget\Categories::class);
}

/**
 * Inicijalizira stvari za admin.
 */
function ep_admin_init()
{
	$services = ep_services();
	/** @var WordPressServiceInterface $wordpress */
	$wordpress = $services->get('WordPress');
	$wordpress->attachAdmin();
}

/**
 * Aktivacija plugina.
 */
function ep_activate()
{
	$services = ep_services();
	/** @var WordPressServiceInterface $wordpress */
	$wordpress = $services->get('WordPress');
	$wordpress->activate();
}

/**
 * Vraća ServiceManager.
 *
 * @param array $configOverride
 *
 * @return ContainerInterface
 */
function ep_services()
{
	static $services;

	if (!isset($services)) {
		$serviceManagerFactory = new \Ep\Core\ServiceManagerFactory();
		$services = $serviceManagerFactory();
	}

	return $services;
}

/**
 * Registrira front-end resurse.
 */
function ep_assets()
{
	$services = ep_services();
	$config = $services->get('config');
	$recaptchaConfig = $config['recaptcha'];

	$manifest = json_decode(file_get_contents(__DIR__ . '/dist/manifest.json'), true);
	wp_enqueue_style('ep-webshop', sprintf('%s/dist/%s', EP_WEBSHOP_URL, $manifest['ep-webshop.css']));
	wp_enqueue_script('ep-webshop', sprintf('%s/dist/%s', EP_WEBSHOP_URL, $manifest['ep-webshop.js']), ['jquery']);
	wp_localize_script('ep-webshop', 'ep_webshop', [
		'ajaxurl' => admin_url('admin-ajax.php'),
		'fileUpload' => [
			'messages' => [
				'fileToBig' => __('The uploaded file is too big!', 'ep-webshop'),
			],
		],
		'recaptcha' => [
			'enabled' => $recaptchaConfig['enabled'] === '1' ? 'yes' : 'no',
			'siteKey' => $recaptchaConfig['site_key'],
			'elements' => [
				'form.register input[type="submit"]',
			],
		],
	]);

	/** @var WordPressServiceInterface $wordpress */
	$wordpress = $services->get('WordPress');
	$wordpress->assets();
}

/**
 * Registrira front-end resurse za admin.
 */
function ep_admin_assets()
{
	$services = ep_services();
	$config = $services->get('config');
	$gathConfig = $config['gath'];

	$manifest = json_decode(file_get_contents(__DIR__ . '/dist/manifest.json'), true);
	$currentPage = $_GET['page'] ?? false;
	$gathEnabled = '1' === $gathConfig['enabled'];

	if ('theme_options' !== $currentPage) {
		wp_enqueue_style('ep-admin', sprintf('%s/dist/%s', EP_WEBSHOP_URL, $manifest['ep-admin.css']));
		wp_enqueue_script('ep-admin', sprintf('%s/dist/%s', EP_WEBSHOP_URL, $manifest['ep-admin.js']), ['jquery']);
		wp_localize_script('ep-admin', 'ep_admin', [
			'gathEnabled' => $gathEnabled ? 'yes' : 'no',
			'gathPrices' => $gathEnabled && in_array('prices', $gathConfig['modules']) ? 'yes' : 'no',
			'gathStock' => $gathEnabled && in_array('stock', $gathConfig['modules']) ? 'yes' : 'no',
			'current_page' => $_GET['page'] ?? false,
		]);
	}


	/** @var WordPressServiceInterface $wordpress */
	$wordpress = $services->get('WordPress');
	$wordpress->adminAssets();
}

/**
 * @param $header
 *
 * @return string
 */
function ep_status_code($header)
{
	$config = get_option('ep_errors', []);

	if (isset($config['error_page']) && !empty($config['error_page'])) {
		$pageId = absint($config['error_page']);

		if (get_queried_object_id() === $pageId) {
			return 'HTTP/1.1 500 Internal Server Error';
		}
	}

	return $header;
}

/**
 * Renderira blok za grešku.
 */
function ep_error_block()
{
	$config = get_option('ep_errors', []);

	if (isset($config['error_block']) && !empty($config['error_block'])) {
		$blockId = absint($config['error_block']);
		$block = get_post($blockId);

		echo apply_filters('the_content', $block->post_content);
	} else {
		echo 'Došlo je do pogreške!';
	}
}