<?php
namespace Ep\Google\Validator;

use GuzzleHttp\ClientInterface;
use Zend\Validator\AbstractValidator;
use Zend\Validator\Exception;

/**
 * Class ReCaptcha
 *
 * @package Ep\Google\Validator
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ReCaptcha extends AbstractValidator
{
	const MISSING_INPUT_SECRET = 'missing-input-secret';
	const INVALID_INPUT_SECRET = 'invalid-input-secret';
	const MISSING_INPUT_RESPONSE = 'missing-input-response';
	const INVALID_INPUT_RESPONSE = 'invalid-input-response';
	const BAD_REQUEST = 'bad-request';

	protected $messageTemplates = [
		self::MISSING_INPUT_SECRET => 'The secret parameter is missing.',
		self::INVALID_INPUT_SECRET => 'The secret parameter is invalid or malformed.',
		self::MISSING_INPUT_RESPONSE => 'The response parameter is missing.',
		self::INVALID_INPUT_RESPONSE => 'The response parameter is invalid or malformed.',
		self::BAD_REQUEST => 'The request is invalid or malformed.',
	];

	/** @var ClientInterface */
	protected $httpClient;
	/** @var string */
	protected $secretKey;

	public function __construct($options = null)
	{
		parent::__construct($options);

		$this->messageTemplates = [
			self::MISSING_INPUT_SECRET => __('The secret parameter is missing.', 'ep-webshop'),
			self::INVALID_INPUT_SECRET => __('The secret parameter is invalid or malformed.', 'ep-webshop'),
			self::MISSING_INPUT_RESPONSE => __('The response parameter is missing.', 'ep-webshop'),
			self::INVALID_INPUT_RESPONSE => __('The response parameter is invalid or malformed.', 'ep-webshop'),
			self::BAD_REQUEST => __('The request is invalid or malformed.', 'ep-webhops'),
		];
	}


	/**
	 * @return ClientInterface
	 */
	public function getHttpClient(): ClientInterface
	{
		return $this->httpClient;
	}

	/**
	 * @param ClientInterface $httpClient
	 */
	public function setHttpClient(ClientInterface $httpClient)
	{
		$this->httpClient = $httpClient;
	}

	/**
	 * @return string
	 */
	public function getSecretKey(): string
	{
		return $this->secretKey;
	}

	/**
	 * @param string $secretKey
	 */
	public function setSecretKey(string $secretKey)
	{
		$this->secretKey = $secretKey;
	}

	/**
	 * Returns true if and only if $value meets the validation requirements
	 *
	 * If $value fails validation, then this method returns false, and
	 * getMessages() will return an array of messages that explain why the
	 * validation failed.
	 *
	 * @param  mixed $value
	 *
	 * @return bool
	 * @throws Exception\RuntimeException If validation of $value is impossible
	 */
	public function isValid($value)
	{
		if (empty($value)) {
			$this->error(self::MISSING_INPUT_RESPONSE);
			return false;
		}

		if (empty($this->secretKey)) {
			$this->error(self::MISSING_INPUT_SECRET);
			return false;
		}

		$response = $this->httpClient->request('POST', '/recaptcha/api/siteverify', [
			'form_params' => [
				'secret' => $this->secretKey,
				'response' => $value,
			],
		]);

		$json = (string) $response->getBody();
		$data = json_decode($json, true);

		if (null === $data) {
			throw new \RuntimeException(sprintf(
				"Došlo je do pogreške prilikom dekodiranja JSON-a: ",
				json_last_error_msg()
			));
		}

		if (true !== $data['success']) {
			foreach ($data['error-codes'] as $errorCode) {
				$this->error($errorCode);
			}
			return false;
		}

		return true;
	}
}