<?php
namespace Ep\InputFilter;

use RuntimeException;
use Traversable;
use Zend\InputFilter\Exception;
use Zend\InputFilter\Factory as BaseFactory;
use Zend\InputFilter\InputFilterInterface;
use Zend\InputFilter\InputInterface;
use Zend\InputFilter\InputProviderInterface;
use Zend\Stdlib\ArrayUtils;
use Zend\Validator\ValidatorChain;

/**
 * Kreira input filtere.
 *
 * U specifikaciju Inputa je dodan ključ 'validator_chain'.
 *
 * @package Ep\InputFilter
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class Factory extends BaseFactory
{
	/**
	 * @param array|Traversable|\Zend\InputFilter\InputFilterProviderInterface $inputFilterSpecification
	 *
	 * @return InputFilterInterface
	 */
	public function createInputFilter($inputFilterSpecification)
	{
		if (is_array($inputFilterSpecification) && !isset($inputFilterSpecification['type'])) {
			$inputFilterSpecification['type'] = InputFilter::class;
		}

		return parent::createInputFilter($inputFilterSpecification);
	}

	/**
	 * Factory for input objects.
	 *
	 * @param  array|Traversable|InputProviderInterface $inputSpecification
	 * @throws Exception\InvalidArgumentException
	 * @throws RuntimeException
	 * @return InputInterface|InputFilterInterface
	 */
	public function createInput($inputSpecification)
	{
		if ($inputSpecification instanceof InputProviderInterface) {
			$inputSpecification = $inputSpecification->getInputSpecification();
		}

		if (! is_array($inputSpecification) && ! $inputSpecification instanceof Traversable) {
			throw new Exception\InvalidArgumentException(sprintf(
				'%s expects an array or Traversable; received "%s"',
				__METHOD__,
				(is_object($inputSpecification) ? get_class($inputSpecification) : gettype($inputSpecification))
			));
		}

		if ($inputSpecification instanceof Traversable) {
			$inputSpecification = ArrayUtils::iteratorToArray($inputSpecification);
		}

		$input = parent::createInput($inputSpecification);

		if (isset($inputSpecification['validator_chain'])) {
			$currentChain = $input->getValidatorChain();
			$newChain = $inputSpecification['validator_chain'];

			if (is_array($newChain)) {
				if (!isset($newChain['name'])) {
					throw new RuntimeException(
						'Invalid validator chain specification provided; does not include "name" key'
					);
				}

				$newChain = $currentChain->plugin($newChain['name'], $newChain['options'] ?? []);
			}

			if (is_string($newChain)) {
				$newChain = $currentChain->plugin($newChain);
			}

			if ($newChain instanceof ValidatorChain) {
				foreach ($currentChain->getValidators() as $validator) {
					$newChain->attach($validator['instance'], $validator['breakChainOnFailure']);
				}

				$input->setValidatorChain($newChain);
			}
		}

		return $input;
	}
}