<?php
namespace Ep\WooCommerce;

use Ep\WordPress\Service\AbstractWordPressService;
use WC_Product;
use WP_Term;

/**
 * Class HideCart
 *
 * @package Ep\WooCommerce
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class HideCart extends AbstractWordPressService
{
	public function attach(): void
	{
		add_filter('woocommerce_loop_add_to_cart_link', [$this, 'addToCartLink'], 10, 2);
		add_filter('woocommerce_is_purchasable', [$this, 'isPurchasable']);
		add_filter('woocommerce_get_price_html', [$this, 'filterPrice']);
		add_filter('woocommerce_format_sale_price', [$this, 'filterPrice']);
		add_filter('woocommerce_variable_price_html', [$this, 'filterPrice']);
	}

	public function attachAdmin(): void
	{
		add_filter('woocommerce_settings_tabs_array', [$this, 'settingsTab'], 100);
		add_action('woocommerce_settings_tabs_ep_woo_hide_cart', [$this, 'settingsTabContent']);
		add_action('woocommerce_update_options', [$this, 'updateSettings']);

	}

	/**
	 * Odlučujemo da li je proizvod moguće kupiti.
	 *
	 * @param $isPurchasable
	 *
	 * @return bool
	 */
	public function isPurchasable($isPurchasable)
	{
		global $product;

		if (!$product instanceof WC_Product) {
			return $isPurchasable;
		}

		return $isPurchasable && !$this->isCartDisabled($product);
	}

	/**
	 * @param $cartIcon
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	public function addToCartLink($cartIcon, WC_Product $product)
	{
		return $this->isCartDisabled($product) ? '' : $cartIcon;
	}

	/**
	 * @param $price
	 *
	 * @return string
	 */
	public function filterPrice($price)
	{
		global $product;
		static $statuses;

		if (!isset($statuses)) {
			$statuses = [];
		}

		if ($product instanceof WC_Product) {
			if ($this->isPriceDisabled($product)) {
				return 'Cijena na upit';
			}
		}

		return $price;
	}

	/**
	 * Vraća true ako je onemogućeno dodavanje proizvoda u košaricu ili je onemogućen prikaz cijene.
	 * Ne možemo kupiti nešto što nema cijene.
	 *
	 * @param WC_Product $product
	 *
	 * @return mixed
	 */
	function isCartDisabled(WC_Product $product)
	{
		static $statuses;

		if (!isset($statuses)) {
			$statuses = [];
		}

		if (!isset($statuses[$product->get_id()])) {
			$hide_cart = get_option('ep_woocommerce_hide_cart', []);
			$hide_cart_product = array_map('trim', explode("\n", get_option('ep_woocommerce_hide_cart_product', '')));

			if (in_array($product->get_sku(''), $hide_cart_product)) {
				$statuses[$product->get_id()] = true || $this->isPriceDisabled($product);
			} else {
				$product_cats = $this->getProductCategoryIds($product);
				$statuses[$product->get_id()] = count(array_intersect($hide_cart, $product_cats)) > 0 || $this->isPriceDisabled($product);
			}
		}

		return $statuses[$product->get_id()];
	}

	/**
	 * Vraća true ako je onemogućen prikaz cijena za dati proizvod.
	 *
	 * @param WC_Product $product
	 *
	 * @return bool
	 */
	protected function isPriceDisabled(WC_Product $product)
	{
		static $statuses;

		if (!isset($statuses)) {
			$statuses = [];
		}

		if (!isset($statuses[$product->get_id()])) {
			$hide_prices = get_option('ep_woocommerce_hide_prices', []);
			$hide_prices_product = array_map('trim', explode("\n", get_option('ep_woocommerce_hide_prices_product', '')));

			if (in_array($product->get_sku(''), $hide_prices_product)) {
				$statuses[$product->get_id()] = true;
			} else {
				$product_cats                   = $this->getProductCategoryIds($product);
				$statuses[ $product->get_id()] = count(array_intersect($hide_prices, $product_cats)) > 0;
			}
		}

		return $statuses[$product->get_id()];
	}

	/**
	 * Vraća sve id-eve kategorija u kojima se proizvod nalazi, direktno i indirektno.
	 *
	 * @param WC_Product $product
	 *
	 * @return array
	 */
	public function getProductCategoryIds(WC_Product $product)
	{
		$ids = [];
		/** @var WP_Term[] $categories */
		$categories = wp_get_post_terms($product->get_id(), 'product_cat');

		foreach ($categories as $category) {
			$ids = array_merge($ids, $this->getTermPath($category));
		}

		$ids = array_unique($ids);
		sort($ids);

		return $ids;
	}

	/**
	 * Vraća sve id-eve od root-a do datog pojma.
	 *
	 * @param WP_Term $term
	 *
	 * @return array
	 */
	protected function getTermPath(WP_Term $term)
	{
		$ids = [];

		while ($term->parent > 0) {
			$ids[] = $term->term_id;
			$term = get_term($term->parent, $term->taxonomy);
		}

		$ids[] = $term->term_id;

		return array_reverse($ids);
	}

	/**
	 * Dodaje tab s postavkama u WooCommerce.
	 *
	 * @param array $tabs
	 *
	 * @return array
	 */
	public function settingsTab(array $tabs)
	{
		$tabs['ep_woo_hide_cart'] = 'Sakrivanje';
		return $tabs;
	}

	/**
	 * Prikazuje obrazac s postavkama.
	 */
	public function settingsTabContent()
	{
		?>
		<table class="form-table">
			<?php woocommerce_admin_fields($this->getSettingsTabFields()); ?>
		</table>
		<?php
	}

	/**
	 * Ažurira postavke.
	 */
	public function updateSettings()
	{
		woocommerce_update_options($this->getSettingsTabFields());
	}

	/**
	 * Vraća polja za obrazac s postavkama.
	 *
	 * @return array
	 */
	protected function getSettingsTabFields()
	{
		static $categories;

		if (!isset($categories)) {
			$categories = $this->getCategories();
		}

		return [
			'title' => [
				'name' => 'Ovdje možete sakriti cijene i dodavanje u košaricu!',
				'type' => 'title',
			],

			'hide_cart' => [
				'id' => 'ep_woocommerce_hide_cart',
				'title' => 'Onemogući dodavanje u košaricu za proizvode u:',
				'type' => 'multiselect',
				'options' => $categories,
				'class' => 'wc-enhanced-select',
			],

			'hide_prices' => [
				'id' => 'ep_woocommerce_hide_prices',
				'title' => 'Onemogući prikaz cijena za proizvode u:',
				'type' => 'multiselect',
				'options' => $categories,
				'class' => 'wc-enhanced-select',
			],

			'hide_cart_product' => [
				'id' => 'ep_woocommerce_hide_cart_product',
				'title' => 'Onemogući dodavanje u košaricu za ove šifre proizvoda:',
				'type' => 'textarea',
				'css' => 'min-height: 250px',
			],

			'hide_prices_product' => [
				'id' => 'ep_woocommerce_hide_prices_product',
				'title' => 'Onemogući prikaz cijena za ove šifre proizvoda:',
				'type' => 'textarea',
				'css' => 'min-height: 250px',
			],
		];
	}

	/**
	 * Dodaje pojam rekurzivno u array s opcijama.
	 *
	 * @param $options
	 * @param WP_Term $term
	 * @param int $level
	 */
	protected function appendTerm(&$options, WP_Term $term, $level = 0)
	{
		$options[$term->term_id] = trim(sprintf('%s %s', str_repeat('-', $level), $term->name));
		/** @var WP_Term[] $children */
		$children = get_terms([
			'taxonomy' => $term->taxonomy,
			'parent' => $term->term_id,
			'hide_empty' => false,
		]);

		foreach ($children as $child) {
			$this->appendTerm($options, $child, $level + 1);
		}
	}

	/**
	 * Vraća sve kategorije proizvoda kao array.
	 *
	 * @return array
	 */
	protected function getCategories()
	{
		$options = [];

		$terms = get_terms([
			'taxonomy' => 'product_cat',
			'parent' => 0,
			'hide_empty' => false,
		]);

		foreach ($terms as $term) {
			$this->appendTerm($options, $term);
		}

		return $options;
	}
}