<?php
namespace Ep\WordPress\Settings;

use Ep\WordPress\Exception\RuntimeException;
use Ep\WordPress\Form\SettingsForm;
use Ep\WordPress\Service\AbstractWordPressService;
use Zend\Stdlib\ArrayUtils;
use Zend\Stdlib\PriorityQueue;

/**
 * Postavke.
 *
 * @package Ep\WordPress\Settings
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class SettingsService extends AbstractWordPressService
{
	/** @var array */
	protected $messages = [
		'error' => [],
		'success' => [],
	];
	/** @var SettingsProviderInterface[] */
	protected $providers;
	/** @var array */
	protected $defaults;
	/** @var PriorityQueue|array */
	protected $tabs;
	/** @var array */
	protected $forms;
	/** @var SettingsForm[] */
	protected $formObjects = [];

	/**
	 * Dodaje handlere na WP akcije i filtere.
	 */
	public function attach(): void
	{
		add_action('admin_menu', [$this, 'addMenuPage'], 20);
	}

	/**
	 * @param SettingsProviderInterface $provider
	 * @return $this
	 */
	public function addProvider(SettingsProviderInterface $provider)
	{
		$this->providers[] = $provider;
		return $this;
	}

	/**
	 * U adminu.
	 */
	public function attachAdmin(): void
	{
		$this->handleForm();
	}

	/**
	 * Dodaje link na stranicu s postavkama u izbornik admina.
	 */
	public function addMenuPage(): void
	{
		add_menu_page(
			'EP Upravljačka ploča',
			'EP Postavke',
			'manage_options',
			'ep-settings',
			[$this, 'renderSettingsPage'],
			'dashicons-forms',
			'2.0'
		);
	}

	/**
	 * Obrađuje POST request.
	 */
	protected function handleForm()
	{
		if ('POST' === $_SERVER['REQUEST_METHOD'] && isset($_GET['page']) && $_GET['page'] === 'ep-settings') {
			if (!isset($_POST['optionKey']) || empty($_POST['optionKey'])) {
				throw new RuntimeException("POST mora sadržavati vrijednost 'optionKey'.");
			}

			$optionKey = trim($_POST['optionKey']);
			$form = $this->getForm($optionKey);

			$form->setData($_POST);

			if ($form->isValid()) {
				$data = (array) $form->getData();
				$defaults = $this->getDefaults($optionKey);

				$option = ArrayUtils::merge($defaults, $data);
				update_option($optionKey, $option);

				add_settings_error('general', 'settings_updated', __('Settings saved.'), 'updated');
			}
		}
	}

	/**
	 * @param string $optionKey
	 *
	 * @return array
	 */
	protected function getDefaults(string $optionKey = null): array
	{
		if (null === $this->defaults) {
			$this->defaults = [];

			foreach ($this->providers as $provider) {
				$this->defaults = ArrayUtils::merge($this->defaults, $provider->getDefaults());
			}
		}

		if (null !== $optionKey) {
			return $this->defaults[$optionKey] ?? [];
		}

		return $this->defaults;
	}

	/**
	 * Vraća array formi.
	 *
	 * Vrijednosti su ključevi formi u ServiceManageru.
	 *
	 * @return array
	 */
	protected function getForms()
	{
		if (null === $this->forms) {
			$this->forms = [];

			foreach ($this->providers as $provider) {
				$this->forms = ArrayUtils::merge($this->forms, $provider->getForms());
			}
		}

		return $this->forms;
	}

	/**
	 * Vraća formu za dato ime opcije.
	 *
	 * @param string $optionKey
	 *
	 * @return SettingsForm
	 */
	protected function getForm(string $optionKey)
	{
		if (!isset($this->formObjects[$optionKey])) {
			$forms = $this->getForms();

			if (!array_key_exists($optionKey, $forms)) {
				throw new \InvalidArgumentException(sprintf(
					"Nepoznata opcija '%s'.", $optionKey
				));
			}

			$formName = $forms[$optionKey];

			if (!$this->services->has($formName)) {
				throw new RuntimeException(sprintf(
					"Forma s imenom '%s' ne postoji u ServiceManageru.", $formName
				));
			}

			$form = $this->services->get($formName);

			if (!$form instanceof SettingsForm) {
				throw new RuntimeException(sprintf(
					"Obrazac za postavke mora biti klase '%s'.", SettingsForm::class
				));
			}

			$option = get_option($optionKey, $this->getDefaults($optionKey));
			$form->setData($option);

			$this->formObjects[$optionKey] = $form;
		}

		return $this->formObjects[$optionKey];
	}

	/**
	 * @return array|PriorityQueue
	 */
	protected function getTabs()
	{
		if (null === $this->tabs) {
			$this->tabs = [];
			$tabs = new PriorityQueue();
			$hasActive = false;

			foreach ($this->providers as $provider) {
				foreach ($provider->getTabs() as $tab) {
					if (!isset($tab['id']) || empty($tab['id'])) {
						throw new \InvalidArgumentException('Tab mora imati id.');
					}

					$tab = ArrayUtils::merge([
						'id' => '',
						'optionKeys' => [],
						'active' => false,
					], $tab);

					if (isset($_POST['optionKey']) && in_array($_POST['optionKey'], $tab['optionKeys'])) {
						$hasActive = true;
						$tab['active'] = true;
					}

					$position = isset($tab['priority']) ? intval($tab['priority']) : 1;
					$tabs->insert($tab, $position);
				}
			}

			foreach ($tabs as $tab) {
				if (!isset($this->tabs[$tab['id']])) {
					$this->tabs[$tab['id']] = $tab;
				} else {
					$this->tabs[$tab['id']] = ArrayUtils::merge($this->tabs[$tab['id']], $tab);
 				}
			}

			if (count($this->tabs) > 0 && !$hasActive) {
				$this->tabs[array_keys($this->tabs)[0]]['active'] = true;
			}
		}

		return $this->tabs;
	}

	/**
	 * Renderira stranicu s postavkama.
	 */
	public function renderSettingsPage()
	{
		$tabs = $this->getTabs();

		foreach ($tabs as &$tab) {
			$tab['forms'] = [];

			foreach ($tab['optionKeys'] as $optionKey) {
				$form = $this->getForm($optionKey);
				$form->get('optionKey')->setValue($optionKey);

				$tab['forms'][] = $form;
			}
		}

		ob_start();
		settings_errors();
		$messages = ob_get_clean();

		echo $this->getView()->render('admin/page-settings.twig', [
			'page_title' => get_admin_page_title(),
			'tabs' => $tabs,
			'messages' => $messages,
		]);
	}
}