<?php
namespace EpTest\InputFilter;

use Ep\InputFilter\Factory;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Zend\Validator\Hostname;
use Zend\Validator\NotEmpty;
use Zend\Validator\ValidatorChain;
use Zend\Validator\ValidatorPluginManager;

/**
 * Testovi za InputFilter Factory.
 *
 * @package EpTest\InputFilter
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class FactoryTest extends TestCase
{
	/**
	 * Ako opcije inputa sadrže ključ 'validator_chain', ValidatorChain
	 * tog inputa će biti zamijenjen novim.
	 */
	public function testCanInjectValidatorChain()
	{
		$replacement = $this->createMock(ValidatorChain::class);

		$factory = new Factory();

		$input = $factory->createInput([
			'name'            => 'test',
			'validator_chain' => $replacement,
		]);

		$replaced = $input->getValidatorChain();

		$this->assertSame($replacement, $replaced);
	}

	/**
	 * Polje 'validator_chain' može sadržavati i string koji je ime Validatora u
	 * ValidatorManageru.
	 */
	public function testCanInjectValidatorChainFromString()
	{
		$replacement = $this->createMock(ValidatorChain::class);
		/** @var ValidatorPluginManager|MockObject $validators */
		$validators = $this->createPartialMock(ValidatorPluginManager::class, ['has', 'get']);
		$validators->method('has')->willReturn(true);
		$validators->method('get')->with('ReplacementChain')->willReturn($replacement);

		$factory = new Factory();
		$chain   = $factory->getDefaultValidatorChain();
		$chain->setPluginManager($validators);

		$input = $factory->createInput([
			'name'            => 'test',
			'validator_chain' => 'ReplacementChain',
		]);

		$replaced = $input->getValidatorChain();

		$this->assertSame($replacement, $replaced);
	}

	/**
	 * U specifikaciji inputa, validator_chain može biti array po kojem možemo kreirati
	 * novi ValidatorChain.
	 */
	public function testCanInjectValidatorChainFromArray()
	{
		$replacement = $this->createMock(ValidatorChain::class);
		/** @var ValidatorPluginManager|MockObject $validators */
		$validators = $this->createPartialMock(ValidatorPluginManager::class, ['has', 'get']);
		$validators->method('has')->with('ReplacementChain')->willReturn(true);
		$validators->method('get')->with('ReplacementChain')->willReturn($replacement);

		$factory = new Factory();
		$chain   = $factory->getDefaultValidatorChain();
		$chain->setPluginManager($validators);

		$input = $factory->createInput([
			'name'            => 'test',
			'validator_chain' => [
				'name' => 'ReplacementChain',
			],
		]);

		$replaced = $input->getValidatorChain();

		$this->assertSame($replacement, $replaced);
	}

	/**
	 * Ako šaljemo validator_chain kao array, mora sadržavati ključ 'name'.
	 *
	 * @expectedException \RuntimeException
	 */
	public function testWillThrowExceptionWhenValidatorChainArrayDoesNotContainName()
	{
		$factory = new Factory();
		$factory->createInput([
			'validator_chain' => [],
		]);
	}

	/**
	 * Ako ValidatorChain zamijenimo nakon što smo već dodali validatore u trenutni chain,
	 * novi chain mora sadržavati iste validatore.
	 */
	public function testNewValidatorChainWillContainValidators()
	{
		$replacement = new ValidatorChain();

		$notEmpty = new NotEmpty();
		$hostname = new Hostname();

		$factory = new Factory();
		$factory->createInput([
			'name'            => 'test',
			'validators'      => [
				$notEmpty,
				$hostname,
			],
			'validator_chain' => $replacement,
		]);

		$validators = $replacement->getValidators();

		$this->assertCount(2, $validators);
		$this->assertInstanceOf(NotEmpty::class, $validators[0]['instance']);
		$this->assertInstanceOf(Hostname::class, $validators[1]['instance']);
	}
}
